@@ -1,7 +1,7 @@
 from importlib.metadata import version
 
-from rhoknp.processors import KNP, Jumanpp, RegexSenter
-from rhoknp.units import BasePhrase, Clause, Document, Morpheme, Phrase, Sentence
+from .processors import KNP, Jumanpp, RegexSenter
+from .units import BasePhrase, Clause, Document, Morpheme, Phrase, Sentence
 
 __version__ = version("rhoknp")
 
