@@ -10,9 +10,7 @@ class Processor(ABC):
     def apply_to_document(self, document: Union[Document, str]) -> Document:
         raise NotImplementedError
 
-    def batch_apply_to_documents(
-        self, documents: Sequence[Union[Document, str]], processes: int = 0
-    ) -> list[Document]:
+    def batch_apply_to_documents(self, documents: Sequence[Union[Document, str]], processes: int = 0) -> list[Document]:
         if processes < 1:
             return list(map(self.apply_to_document, documents))
         with Pool(processes=processes) as pool:
@@ -22,9 +20,7 @@ class Processor(ABC):
     def apply_to_sentence(self, sentence: Union[Sentence, str]) -> Sentence:
         raise NotImplementedError
 
-    def batch_apply_to_sentences(
-        self, sentences: Sequence[Union[Sentence, str]], processes: int = 0
-    ) -> list[Sentence]:
+    def batch_apply_to_sentences(self, sentences: Sequence[Union[Sentence, str]], processes: int = 0) -> list[Sentence]:
         if processes < 1:
             return list(map(self.apply, sentences))
         with Pool(processes=processes) as pool:
@@ -33,7 +29,5 @@ class Processor(ABC):
     def apply(self, sentence: Union[Sentence, str]) -> Sentence:
         return self.apply_to_sentence(sentence)
 
-    def batch_apply(
-        self, sentences: Sequence[Union[Sentence, str]], processes: int = 0
-    ) -> list[Sentence]:
-        return self.batch_apply_to_sentences(sentences)
+    def batch_apply(self, sentences: Sequence[Union[Sentence, str]], processes: int = 0) -> list[Sentence]:
+        return self.batch_apply_to_sentences(sentences, processes)
