@@ -3,13 +3,17 @@ import re
 from functools import cached_property
 from typing import TYPE_CHECKING, Optional
 
-from rhoknp.rel.coreference import Entity, EntityManager
-from rhoknp.rel.exophora import ExophoraReferent
-from rhoknp.rel.pas import CaseInfoFormat, Pas
-from rhoknp.rel.predicate import Predicate
+from rhoknp.cohesion.coreference import Entity, EntityManager
+from rhoknp.cohesion.discourse_relation import DiscourseRelationTag
+from rhoknp.cohesion.exophora import ExophoraReferent
+from rhoknp.cohesion.pas import CaseInfoFormat, Pas
+from rhoknp.cohesion.predicate import Predicate
+from rhoknp.cohesion.rel import RelMode, RelTag, RelTagList
+from rhoknp.props.dependency import DepType
+from rhoknp.props.feature import FeatureDict
+from rhoknp.props.named_entity import NETagList
 from rhoknp.units.morpheme import Morpheme
 from rhoknp.units.unit import Unit
-from rhoknp.units.utils import DepType, Features, Rel, RelMode, Rels
 from rhoknp.utils.constants import ALL_CASES, ALL_COREFS
 
 if TYPE_CHECKING:
@@ -29,7 +33,15 @@ class BasePhrase(Unit):
     )
     count = 0
 
-    def __init__(self, parent_index: Optional[int], dep_type: Optional[DepType], features: Features, rels: Rels):
+    def __init__(
+        self,
+        parent_index: Optional[int],
+        dep_type: Optional[DepType],
+        features: FeatureDict,
+        rels: RelTagList,
+        ne_tags: NETagList,
+        discourse_relation_tag: DiscourseRelationTag,
+    ):
         super().__init__()
 
         # parent unit
@@ -40,8 +52,10 @@ class BasePhrase(Unit):
 
         self.parent_index: Optional[int] = parent_index  #: 係り先の基本句の文内におけるインデックス．
         self.dep_type: Optional[DepType] = dep_type  #: 係り受けの種類．
-        self.features: Features = features  #: 素性．
-        self.rels: Rels = rels  #: 基本句間関係．
+        self.features: FeatureDict = features  #: 素性．
+        self.rels: RelTagList = rels  #: 基本句間関係．
+        self.ne_tags: NETagList = ne_tags  #: 固有表現タグ．
+        self.discourse_relation_tag: DiscourseRelationTag = discourse_relation_tag  #: 談話関係タグ．
         self.pas: Optional["Pas"] = None  #: 述語項構造．
         self.entities: set[Entity] = set()  #: 参照しているエンティティ．
         self.entities_nonidentical: set[Entity] = set()  #: ≒で参照しているエンティティ．
@@ -49,13 +63,14 @@ class BasePhrase(Unit):
         self.index = self.count
         BasePhrase.count += 1
 
-    @property
+    @cached_property
     def global_index(self) -> int:
         """文書全体におけるインデックス．"""
-        offset = 0
-        for prev_sentence in self.document.sentences[: self.sentence.index]:
-            offset += len(prev_sentence.base_phrases)
-        return self.index + offset
+        if self.index > 0:
+            return self.sentence.base_phrases[self.index - 1].global_index + 1
+        if self.sentence.index == 0:
+            return self.index
+        return self.document.sentences[self.sentence.index - 1].base_phrases[-1].global_index + 1
 
     @property
     def parent_unit(self) -> Optional["Phrase"]:
@@ -116,17 +131,17 @@ class BasePhrase(Unit):
 
     @property
     def morphemes(self) -> list[Morpheme]:
-        """形態素．"""
+        """形態素のリスト．"""
         if self._morphemes is None:
             raise AssertionError
         return self._morphemes
 
     @morphemes.setter
     def morphemes(self, morphemes: list[Morpheme]) -> None:
-        """形態素．
+        """形態素のリスト．
 
         Args:
-            morphemes: 形態素．
+            morphemes: 形態素のリスト．
         """
         for morpheme in morphemes:
             morpheme.base_phrase = self
@@ -187,9 +202,11 @@ class BasePhrase(Unit):
             raise ValueError(f"malformed line: {first_line}")
         parent_index = int(match.group("pid")) if match.group("pid") is not None else None
         dep_type = DepType(match.group("dtype")) if match.group("dtype") is not None else None
-        features = Features.from_fstring(match.group("tags") or "")
-        rels = Rels.from_fstring(match.group("tags") or "")
-        base_phrase = cls(parent_index, dep_type, features, rels)
+        features = FeatureDict.from_fstring(match.group("tags") or "")
+        rels = RelTagList.from_fstring(match.group("tags") or "")
+        ne_tags = NETagList.from_fstring(match.group("tags") or "")
+        discourse_relation_tag = DiscourseRelationTag.from_fstring(match.group("tags") or "")
+        base_phrase = cls(parent_index, dep_type, features, rels, ne_tags, discourse_relation_tag)
 
         morphemes: list[Morpheme] = []
         for line in lines:
@@ -205,8 +222,12 @@ class BasePhrase(Unit):
         if self.parent_index is not None:
             assert self.dep_type is not None
             ret += f" {self.parent_index}{self.dep_type.value}"
-        if self.rels or self.features:
-            ret += f" {self.rels.to_fstring()}{self.features.to_fstring()}"
+        if self.rels or self.ne_tags or self.features or self.discourse_relation_tag:
+            ret += " "
+            ret += self.rels.to_fstring()
+            ret += self.ne_tags.to_fstring()
+            ret += self.features.to_fstring()
+            ret += self.discourse_relation_tag.to_fstring()
         ret += "\n"
         ret += "".join(morpheme.to_jumanpp() for morpheme in self.morphemes)
         return ret
@@ -251,7 +272,7 @@ class BasePhrase(Unit):
             self.entities.add(entity)
         entity.add_mention(self, nonidentical=nonidentical)
 
-    def parse_rel(self) -> None:
+    def parse_rel_tag(self) -> None:
         """関係タグ付きコーパスにおける <rel> タグをパース．"""
         if self.pas is None:
             self.pas = Pas(Predicate(self))
@@ -267,7 +288,7 @@ class BasePhrase(Unit):
             else:
                 logger.warning(f"unknown rel type: {rel.type}")
 
-    def parse_knp_pas(self) -> None:
+    def parse_pas_tag(self) -> None:
         """KNP 解析結果における <述語項構造> タグおよび <格解析結果> タグをパース．"""
         if "述語項構造" in self.features:
             pas_string = self.features["述語項構造"]
@@ -281,13 +302,7 @@ class BasePhrase(Unit):
             pas = Pas(Predicate(self))
         self.pas = pas
 
-    def reset_rels(self) -> None:
-        """PAS と共参照関係をリセット．"""
-        self.pas = None
-        for entity in self.entities_all:
-            entity.remove_mention(self)
-
-    def _add_pas(self, rel: Rel) -> None:
+    def _add_pas(self, rel: RelTag) -> None:
         """述語項構造を追加．"""
         entity_manager: EntityManager = self.document.entity_manager
         assert self.pas is not None
@@ -305,7 +320,7 @@ class BasePhrase(Unit):
             entity = entity_manager.get_or_create_entity(ExophoraReferent(rel.target))
             self.pas.add_special_argument(rel.type, rel.target, eid=entity.eid, mode=rel.mode)
 
-    def _add_coreference(self, rel: Rel) -> None:
+    def _add_coreference(self, rel: RelTag) -> None:
         """共参照関係を追加．"""
         entity_manager: EntityManager = self.document.entity_manager
         # create source entity
@@ -331,7 +346,7 @@ class BasePhrase(Unit):
                 target_entity = entity_manager.get_or_create_entity(exophora_referent=ExophoraReferent(rel.target))
                 entity_manager.merge_entities(self, None, source_entity, target_entity, nonidentical)
 
-    def _get_target_base_phrase(self, rel: Rel) -> Optional["BasePhrase"]:
+    def _get_target_base_phrase(self, rel: RelTag) -> Optional["BasePhrase"]:
         """rel が指す基本句を取得．"""
         sentences = [sent for sent in self.document.sentences if sent.sid == rel.sid]
         if not sentences:
