@@ -1,15 +1,16 @@
 import logging
 from typing import Any, Optional, Sequence, Union
 
-from rhoknp.rel.coreference import EntityManager
-from rhoknp.rel.pas import Pas
+from rhoknp.cohesion.coreference import EntityManager
+from rhoknp.cohesion.pas import Pas
+from rhoknp.props.named_entity import NamedEntity
 from rhoknp.units.base_phrase import BasePhrase
 from rhoknp.units.clause import Clause
 from rhoknp.units.morpheme import Morpheme
 from rhoknp.units.phrase import Phrase
 from rhoknp.units.sentence import Sentence
 from rhoknp.units.unit import Unit
-from rhoknp.units.utils import is_comment_line
+from rhoknp.utils.utils import is_comment_line
 
 logger = logging.getLogger(__name__)
 
@@ -42,17 +43,14 @@ class Document(Unit):
 
         self.entity_manager = EntityManager()
 
-        self.has_error = False
-
     def _post_init(self) -> None:
         """インスタンス作成後の追加処理を行う．"""
-        if self._sentences:
-            for sentence in self._sentences:
+        if self.need_senter is False:
+            for sentence in self.sentences:
                 sentence.post_init()
         if self.need_knp is False:
-            self._parse_rel()
-        if self.need_clause_tag is False:
-            self._parse_discourse_relation()
+            for base_phrase in self.base_phrases:
+                base_phrase.parse_rel_tag()
 
     @property
     def parent_unit(self) -> None:
@@ -122,6 +120,11 @@ class Document(Unit):
         """
         return [morpheme for sentence in self.sentences for morpheme in sentence.morphemes]
 
+    @property
+    def named_entities(self) -> list[NamedEntity]:
+        """固有表現のリスト．"""
+        return [ne for sentence in self.sentences for ne in sentence.named_entities]
+
     @property
     def need_senter(self) -> bool:
         """文分割がまだなら True．"""
@@ -160,7 +163,7 @@ class Document(Unit):
             text = "天気が良かったので散歩した。途中で先生に会った。"
             doc = Document.from_raw_text(text)
         """
-        document = cls(text)
+        document = cls(text.strip())
         document._post_init()
         return document
 
@@ -226,7 +229,7 @@ class Document(Unit):
                 else:
                     sentences_.append(Sentence.from_knp(sentence.to_knp(), post_init=False))
             else:
-                sentences_.append(Sentence.from_raw_text(sentence, post_init=False))
+                sentences_.append(Sentence.from_raw_text(sentence.strip(), post_init=False))
         document.sentences = sentences_
         if sentences_:
             document.doc_id = sentences_[0].doc_id
@@ -266,7 +269,7 @@ class Document(Unit):
             doc = Document.from_jumanpp(jumanpp_text)
 
         .. note::
-            複数文の解析結果が含まれている場合，一つの文書として扱われる．．
+            複数文の解析結果が含まれている場合，一つの文書として扱われる．
         """
         document = cls()
         sentences = []
@@ -279,7 +282,6 @@ class Document(Unit):
                 try:
                     sentences.append(Sentence.from_jumanpp("\n".join(sentence_lines) + "\n", post_init=False))
                 except Exception as e:
-                    document.has_error = True
                     if not ignore_errors:
                         raise e
                 sentence_lines = []
@@ -349,7 +351,6 @@ class Document(Unit):
                 try:
                     sentences.append(Sentence.from_knp("\n".join(sentence_lines) + "\n", post_init=False))
                 except Exception as e:
-                    document.has_error = True
                     if not ignore_errors:
                         raise e
                 sentence_lines = []
@@ -359,6 +360,20 @@ class Document(Unit):
         document._post_init()
         return document
 
+    def reparse(self) -> "Document":
+        """文書を再構築．
+
+        .. note::
+            解析結果に対する編集を有効にする際に実行する必要がある．
+        """
+        if self.need_knp is False:
+            return Document.from_knp(self.to_knp())
+        elif self.need_jumanpp is False:
+            return Document.from_jumanpp(self.to_jumanpp())
+        elif self.need_senter is False:
+            return Document.from_line_by_line_text(self.to_plain())
+        return Document.from_raw_text(self.to_plain())
+
     def to_plain(self) -> str:
         """プレーンテキストフォーマットに変換．
 
@@ -377,23 +392,6 @@ class Document(Unit):
         """KNP フォーマットに変換．"""
         return "".join(sentence.to_knp() for sentence in self.sentences)
 
-    def _parse_rel(self) -> None:
-        """関係タグ付きコーパスにおける <rel> タグをパース．"""
-        for base_phrase in self.base_phrases:
-            base_phrase.parse_rel()
-
-    def reparse_rel(self) -> None:
-        """base_phrases の持つ rel に基づき PAS と共参照関係を再構築．"""
-        for base_phrase in self.base_phrases:
-            base_phrase.reset_rels()
-        self.entity_manager.reset()
-        self._parse_rel()
-
-    def _parse_discourse_relation(self) -> None:
-        """<談話関係> タグをパース．"""
-        for clause in self.clauses:
-            clause.parse_discourse_relation()
-
     def __eq__(self, other: Any) -> bool:
         if isinstance(other, Document) is False:
             return False
