@@ -15,11 +15,11 @@ class Unit(ABC):
         return f"{self.__class__.__name__}(index={repr(self.index)}, text={repr(self.text)})"
 
     def __eq__(self, other: Any) -> bool:
-        if isinstance(other, type(self)):
-            if self.parent_unit != other.parent_unit:
-                return False
-            return self.index == other.index
-        return False
+        if isinstance(other, type(self)) is False:
+            return False
+        if self.parent_unit != other.parent_unit:
+            return False
+        return self.index == other.index
 
     @property
     def parent_unit(self) -> Optional["Unit"]:
@@ -41,9 +41,10 @@ class Unit(ABC):
 
     @property
     def text(self) -> str:
-        if self.child_units is not None:
-            return "".join(str(child_unit) for child_unit in self.child_units)
-        elif self._text is not None:
+        if self._text is not None:
+            return self._text
+        elif self.child_units is not None:
+            self._text = "".join(str(child_unit) for child_unit in self.child_units)
             return self._text
         raise AssertionError
 
